import json
from .db import PUT_TASK_QUERY
from .base import Base


class Master(Base):
    async def publish(self, queue_name, **kwargs):
        async with self.pool.acquire() as conn:
            await self.ensure_table_exists(conn)
            await conn.execute(PUT_TASK_QUERY, queue_name, json.dumps(kwargs))



from postgres_queue_artgas import Master, Worker
import asyncpg
import asyncio


async def task_handler(string1, string2):
    print(string1, string2)


async def main():
    pool = await asyncpg.create_pool(database='postgres',
                                     user='postgres',
                                     password='k3k1n6_0nl1n3',
                                     )
    master = Master(pool)
    await master.publish('test', string1='a', string2='b')
    worker = Worker(pool=pool)
    await worker.consume('test', task_handler)
    await asyncio.Future()


asyncio.run(main())



import asyncio
import json
from .base import Base
from typing import Callable, Awaitable
from .db import QUERY_TAKE_TASK, UPDATE_FINISHED_TASK_QUERY


class Worker(Base):
    def __init__(self, *, poll_interval: float = 0.5, **kwargs):
        super().__init__(**kwargs)
        self.poll_interval = poll_interval

    async def consume(self, queue_name: str, handler: Callable[[...], Awaitable[None]]):
        while True:
            async with self.pool.acquire() as conn:
                await self.ensure_table_exists(conn)
                task = await conn.fetchrow(QUERY_TAKE_TASK, queue_name)
            if not task:
                await asyncio.sleep(self.poll_interval)
                continue
            task_id = task['id']
            task_data = json.loads(task["data"])
            try:
                await handler(**task_data)
            finally:
                async with self.pool.acquire() as conn:
                    await conn.fetch(UPDATE_FINISHED_TASK_QUERY, task_id)



CREATE_TABLE_QUERY = '''
create table if not exists tasks 
(
id bigserial not null
    constraint task_pkey
        primary key,
queue_name text not null,
data json,
finished boolean default false,
started_at timestamp default CURRENT_TIMESTAMP
);'''

PUT_TASK_QUERY = '''
insert into tasks (queue_name, data) values ($1, $2)
'''

QUERY_TAKE_TASK = '''
with next_task as (
select id, data from tasks
where queue_name = $1 and 
(started_at IS NULL or started_at < current_timestamp - interval '5 seconds' and finished = false)
limit 1
for update skip locked
)
update tasks
set
    started_at = current_timestamp
from next_task
where tasks.id = next_task.id
returning tasks.id, tasks.data;'''

UPDATE_FINISHED_TASK_QUERY = '''
update tasks set finished = true where id = $1
'''



from setuptools import setup, find_packages

setup(name="postgres_queue_artgas",
      version="1.0.0",
      packages=find_packages(exclude=['tests']),
      description="Custom PostgreSQL Queue",
      author_email="kek@mail.ru",
      zip_safe=False)



from .db import CREATE_TABLE_QUERY


class Base:
    def __init__(self, pool):
        self.pool = pool
        self.table_exists = False

    async def ensure_table_exists(self, conn):
        if not self.table_exists:
            await conn.execute(CREATE_TABLE_QUERY)
            self.table_exists = True
